@@ -1,6 +1,4 @@
-#!/usr/bin/python
-
-# Copyright (C) 2003-2005 Robey Pointer <robey@lag.net>
+# Copyright (C) 2003-2006 Robey Pointer <robey@lag.net>
 #
 # This file is part of paramiko.
 #
@@ -23,8 +21,8 @@ L{ServerInterface} is an interface to override for server support.
 """
 
 import threading
-from common import *
-import util
+from paramiko.common import *
+from paramiko import util
 
 
 class InteractiveQuery (object):
@@ -445,8 +443,6 @@ class SubsystemHandler (threading.Thread):
     authenticated and requests subsytem C{"mp3"}, an object of class
     C{MP3Handler} will be created, and L{start_subsystem} will be called on
     it from a new thread.
-
-    @since: ivysaur
     """
     def __init__(self, channel, name, server):
         """
