@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2005 Robey Pointer <robey@lag.net>
+# Copyright (C) 2003-2006 Robey Pointer <robey@lag.net>
 #
 # This file is part of paramiko.
 #
@@ -16,14 +16,21 @@
 # along with Paramiko; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 
-from __future__ import generators
-
 """
 Useful functions used by the rest of paramiko.
 """
 
-import sys, struct, traceback, threading
-from common import *
+from __future__ import generators
+
+from binascii import hexlify, unhexlify
+import sys
+import struct
+import traceback
+import threading
+
+from paramiko.common import *
+from paramiko.config import SSHConfig
+
 
 # Change by RogerB - python < 2.3 doesn't have enumerate so we implement it
 if sys.version_info < (2,3):
@@ -36,6 +43,7 @@ if sys.version_info < (2,3):
                 yield (count, item)
                 count += 1
 
+
 def inflate_long(s, always_positive=False):
     "turns a normalized byte string into a long-int (adapted from Crypto.Util.number)"
     out = 0L
@@ -108,12 +116,10 @@ def format_binary_line(data):
     return '%-50s %s' % (left, right)
 
 def hexify(s):
-    "turn a string into a hex sequence"
-    return ''.join(['%02X' % ord(c) for c in s])
+    return hexlify(s).upper()
 
 def unhexify(s):
-    "turn a hex sequence back into a string"
-    return ''.join([chr(int(s[i:i+2], 16)) for i in range(0, len(s), 2)])
+    return unhexlify(s)
 
 def safe_string(s):
     out = ''
@@ -161,12 +167,12 @@ def generate_key_bytes(hashclass, salt, key, nbytes):
     if len(salt) > 8:
         salt = salt[:8]
     while nbytes > 0:
-        hash = hashclass.new()
+        hash_obj = hashclass.new()
         if len(digest) > 0:
-            hash.update(digest)
-        hash.update(key)
-        hash.update(salt)
-        digest = hash.digest()
+            hash_obj.update(digest)
+        hash_obj.update(key)
+        hash_obj.update(salt)
+        digest = hash_obj.digest()
         size = min(nbytes, len(digest))
         keydata += digest[:size]
         nbytes -= size
@@ -182,35 +188,29 @@ def load_host_keys(filename):
     This type of file unfortunately doesn't exist on Windows, but on posix,
     it will usually be stored in C{os.path.expanduser("~/.ssh/known_hosts")}.
     
+    Since 1.5.3, this is just a wrapper around L{HostKeys}.
+
     @param filename: name of the file to read host keys from
     @type filename: str
     @return: dict of host keys, indexed by hostname and then keytype
     @rtype: dict(hostname, dict(keytype, L{PKey <paramiko.pkey.PKey>}))
     """
-    import base64
-    from rsakey import RSAKey
-    from dsskey import DSSKey
-    
-    keys = {}
-    f = file(filename, 'r')
-    for line in f:
-        line = line.strip()
-        if (len(line) == 0) or (line[0] == '#'):
-            continue
-        keylist = line.split(' ')
-        if len(keylist) != 3:
-            continue
-        hostlist, keytype, key = keylist
-        hosts = hostlist.split(',')
-        for host in hosts:
-            if not keys.has_key(host):
-                keys[host] = {}
-            if keytype == 'ssh-rsa':
-                keys[host][keytype] = RSAKey(data=base64.decodestring(key))
-            elif keytype == 'ssh-dss':
-                keys[host][keytype] = DSSKey(data=base64.decodestring(key))
-    f.close()
-    return keys
+    from paramiko.hostkeys import HostKeys
+    return HostKeys(filename)
+
+def parse_ssh_config(file_obj):
+    """
+    Provided only as a backward-compatible wrapper around L{SSHConfig}.
+    """
+    config = SSHConfig()
+    config.parse(file_obj)
+    return config
+
+def lookup_ssh_host_config(hostname, config):
+    """
+    Provided only as a backward-compatible wrapper around L{SSHConfig}.
+    """
+    return config.lookup(hostname)
 
 def mod_inverse(x, m):
     # it's crazy how small python can make this function.
@@ -251,15 +251,20 @@ def log_to_file(filename, level=DEBUG):
     l.setLevel(level)
     f = open(filename, 'w')
     lh = logging.StreamHandler(f)
-    lh.setFormatter(logging.Formatter('%(levelname)-.3s [%(asctime)s] thr=%(_threadid)-3d %(name)s: %(message)s',
+    lh.setFormatter(logging.Formatter('%(levelname)-.3s [%(asctime)s.%(msecs)03d] thr=%(_threadid)-3d %(name)s: %(message)s',
                                       '%Y%m%d-%H:%M:%S'))
     l.addHandler(lh)
 
+# make only one filter object, so it doesn't get applied more than once
+class PFilter (object):
+    def filter(self, record):
+        record._threadid = get_thread_id()
+        return True
+_pfilter = PFilter()
+
 def get_logger(name):
     l = logging.getLogger(name)
-    class PFilter (object):
-        def filter(self, record):
-            record._threadid = get_thread_id()
-            return True
-    l.addFilter(PFilter())
+    l.addFilter(_pfilter)
     return l
+
+
