@@ -20,12 +20,17 @@
 Some unit tests for the ssh2 protocol in Transport.
 """
 
-import sys, time, threading, unittest
+from binascii import hexlify, unhexlify
 import select
+import sys
+import time
+import threading
+import unittest
+
 from paramiko import Transport, SecurityOptions, ServerInterface, RSAKey, DSSKey, \
-    SSHException, BadAuthenticationType, InteractiveQuery, util
+    SSHException, BadAuthenticationType, InteractiveQuery, ChannelException
 from paramiko import AUTH_FAILED, AUTH_PARTIALLY_SUCCESSFUL, AUTH_SUCCESSFUL
-from paramiko import OPEN_SUCCEEDED
+from paramiko import OPEN_SUCCEEDED, OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED
 from loop import LoopSocket
 
 
@@ -81,6 +86,8 @@ class NullServer (ServerInterface):
         return AUTH_FAILED
 
     def check_channel_request(self, kind, chanid):
+        if kind == 'bogus':
+            return OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED
         return OPEN_SUCCEEDED
 
     def check_channel_exec_request(self, channel, command):
@@ -130,11 +137,11 @@ class TransportTest (unittest.TestCase):
             
     def test_2_compute_key(self):
         self.tc.K = 123281095979686581523377256114209720774539068973101330872763622971399429481072519713536292772709507296759612401802191955568143056534122385270077606457721553469730659233569339356140085284052436697480759510519672848743794433460113118986816826624865291116513647975790797391795651716378444844877749505443714557929L
-        self.tc.H = util.unhexify('0C8307CDE6856FF30BA93684EB0F04C2520E9ED3')
+        self.tc.H = unhexlify('0C8307CDE6856FF30BA93684EB0F04C2520E9ED3')
         self.tc.session_id = self.tc.H
         key = self.tc._compute_key('C', 32)
         self.assertEquals('207E66594CA87C44ECCBA3B3CD39FDDB378E6FDB0F97C54B2AA0CFBF900CD995',
-            util.hexify(key))
+                          hexlify(key).upper())
 
     def test_3_simple(self):
         """
@@ -189,7 +196,7 @@ class TransportTest (unittest.TestCase):
         self.assertEquals(12, self.tc.packetizer.get_mac_size_in())
         
         self.tc.send_ignore(1024)
-        self.assert_(self.tc.renegotiate_keys())
+        self.tc.renegotiate_keys()
         self.ts.send_ignore(1024)
 
     def test_5_keepalive(self):
@@ -256,7 +263,7 @@ class TransportTest (unittest.TestCase):
             self.assert_(False)
         except:
             etype, evalue, etb = sys.exc_info()
-            self.assertEquals(SSHException, etype)
+            self.assert_(issubclass(etype, SSHException))
         self.tc.auth_password(username='slowdive', password='pygmalion')
         event.wait(1.0)
         self.assert_(event.isSet())
@@ -351,10 +358,14 @@ class TransportTest (unittest.TestCase):
 
         chan = self.tc.open_session()
         schan = self.ts.accept(1.0)
-        self.assert_(not chan.exec_command('no'))
+        try:
+            chan.exec_command('no')
+            self.assert_(False)
+        except SSHException, x:
+            pass
         
         chan = self.tc.open_session()
-        self.assert_(chan.exec_command('yes'))
+        chan.exec_command('yes')
         schan = self.ts.accept(1.0)
         schan.send('Hello there.\n')
         schan.send_stderr('This is on stderr.\n')
@@ -369,7 +380,7 @@ class TransportTest (unittest.TestCase):
         
         # now try it with combined stdout/stderr
         chan = self.tc.open_session()
-        self.assert_(chan.exec_command('yes'))
+        chan.exec_command('yes')
         schan = self.ts.accept(1.0)
         schan.send('Hello there.\n')
         schan.send_stderr('This is on stderr.\n')
@@ -400,7 +411,7 @@ class TransportTest (unittest.TestCase):
         self.assert_(self.ts.is_active())
 
         chan = self.tc.open_session()
-        self.assert_(chan.invoke_shell())
+        chan.invoke_shell()
         schan = self.ts.accept(1.0)
         chan.send('communist j. cat\n')
         f = schan.makefile()
@@ -408,7 +419,31 @@ class TransportTest (unittest.TestCase):
         chan.close()
         self.assertEquals('', f.readline())
 
-    def test_D_exit_status(self):
+    def test_D_channel_exception(self):
+        """
+        verify that ChannelException is thrown for a bad open-channel request.
+        """
+        host_key = RSAKey.from_private_key_file('tests/test_rsa.key')
+        public_host_key = RSAKey(data=str(host_key))
+        self.ts.add_server_key(host_key)
+        event = threading.Event()
+        server = NullServer()
+        self.assert_(not event.isSet())
+        self.ts.start_server(event, server)
+        self.tc.ultra_debug = True
+        self.tc.connect(hostkey=public_host_key)
+        self.tc.auth_password(username='slowdive', password='pygmalion')
+        event.wait(1.0)
+        self.assert_(event.isSet())
+        self.assert_(self.ts.is_active())
+
+        try:
+            chan = self.tc.open_channel('bogus')
+            self.fail('expected exception')
+        except ChannelException, x:
+            self.assert_(x.code == OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED)
+
+    def test_E_exit_status(self):
         """
         verify that get_exit_status() works.
         """
@@ -428,7 +463,7 @@ class TransportTest (unittest.TestCase):
 
         chan = self.tc.open_session()
         schan = self.ts.accept(1.0)
-        self.assert_(chan.exec_command('yes'))
+        chan.exec_command('yes')
         schan.send('Hello there.\n')
         # trigger an EOF
         schan.shutdown_read()
@@ -442,7 +477,7 @@ class TransportTest (unittest.TestCase):
         self.assertEquals(23, chan.recv_exit_status())
         chan.close()
 
-    def test_E_select(self):
+    def test_F_select(self):
         """
         verify that select() on a channel works.
         """
@@ -461,7 +496,7 @@ class TransportTest (unittest.TestCase):
         self.assert_(self.ts.is_active())
 
         chan = self.tc.open_session()
-        self.assert_(chan.invoke_shell())
+        chan.invoke_shell()
         schan = self.ts.accept(1.0)
 
         # nothing should be ready        
@@ -505,7 +540,7 @@ class TransportTest (unittest.TestCase):
         
         chan.close()
    
-    def test_F_renegotiate(self):
+    def test_G_renegotiate(self):
         """
         verify that a transport can correctly renegotiate mid-stream.
         """
@@ -524,7 +559,7 @@ class TransportTest (unittest.TestCase):
         self.tc.packetizer.REKEY_BYTES = 16384
         
         chan = self.tc.open_session()
-        self.assert_(chan.exec_command('yes'))
+        chan.exec_command('yes')
         schan = self.ts.accept(1.0)
 
         self.assertEquals(self.tc.H, self.tc.session_id)
@@ -541,7 +576,7 @@ class TransportTest (unittest.TestCase):
 
         schan.close()
 
-    def test_G_compression(self):
+    def test_H_compression(self):
         """
         verify that zlib compression is basically working.
         """
@@ -560,7 +595,7 @@ class TransportTest (unittest.TestCase):
         self.assert_(self.ts.is_active())
 
         chan = self.tc.open_session()
-        self.assert_(chan.exec_command('yes'))
+        chan.exec_command('yes')
         schan = self.ts.accept(1.0)
 
         bytes = self.tc.packetizer._Packetizer__sent_bytes
@@ -568,6 +603,7 @@ class TransportTest (unittest.TestCase):
         bytes2 = self.tc.packetizer._Packetizer__sent_bytes
         # tests show this is actually compressed to *52 bytes*!  including packet overhead!  nice!! :)
         self.assert_(bytes2 - bytes < 1024)
+        self.assertEquals(52, bytes2 - bytes)
 
         chan.close()
         schan.close()
